window.Gleam.initSearch([{"doc":"barnacle","title":"barnacle","content":"# Barnacle\n\n[![Package Version](https://img.shields.io/hexpm/v/barnacle)](https://hex.pm/packages/barnacle)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/barnacle/)\n\nSelf-healing clusters for Gleam applications on the BEAM!\n\nConnect to other BEAM nodes, and automatically reconnect when they go down.\n\n## Features\n\n- Discover nodes using different strategies. Built-in strategies:\n  - Local EPMD\n  - Remote EPMD\n  - DNS\n- Automatically reconnect to nodes when they come back online\n- Supply your own strategies\n- Listen to events\n- Trigger refreshes manually\n\n## Getting Started\n\n```sh\ngleam add barnacle\n```\n\nYou&#39;ll also need to start your Gleam process with a node name, and set the cookie.\nYou can do this using the `ERL_FLAGS` environment variable.\n\n&gt; [!NOTE]\n&gt; There&#39;s a Fly.io example coming soon.\n\n```sh\nERL_FLAGS=&quot;-name my_app@127.0.0.1 -setcookie my_cookie&quot; gleam run\n```\n\n### Start Barnacle under a Supervisor (recommended)\n\n```gleam\nimport barnacle\nimport gleam/erlang/process\nimport gleam/otp/supervisor\n\npub fn main() {\n  // Configure your Barnacle\n  let barnacle =\n    barnacle.local_epmd()\n    |&gt; barnacle.with_poll_interval(15_000)\n    |&gt; barnacle.with_name(&quot;my_barnacle&quot;)\n\n  // Create a process to receive the child process later\n  let self = process.new_subject()\n\n  // Start the child process under a supervisor\n  let barnacle_worker = barnacle.child_spec(barnacle, None)\n  let assert Ok(_) = supervisor.start(supervisor.add(_, barnacle_worker))\n\n  // Get a subject to send messages to the child process\n  let assert Ok(barnacle_subject) = process.receive(self, 10_000)\n\n  // Continue your setup...\n\n  process.sleep_forever()\n}\n```\n\n### Start Barnacle as a standalone actor\n\nThis is **not** recommended as your barnacle won&#39;t be restarted if it crashes for any\nreason.\n\n```gleam\nimport barnacle\nimport gleam/erlang/process\n\npub fn main() {\n  // Configure your Barnacle\n  let barnacle =\n    barnacle.local_epmd()\n    |&gt; barnacle.with_poll_interval(15_000)\n    |&gt; barnacle.with_name(&quot;my_barnacle&quot;)\n\n  // Start the actor\n  let barnacle_subject = barnacle.start(barnacle)\n\n  // Continue your setup...\n\n  process.sleep_forever()\n}\n```\n\n### Run the refresh function once\n\nIf you don&#39;t need any sort of polling, or want to manage your the refresh lifecycle\nyourself, you can call the `run_once` function to run the refresh function a single\ntime.\n\nIn this case, most of the configuration options will be ignored.\n\n```gleam\nimport barnacle\n\npub fn main() {\n  // Configure your Barnacle\n  let barnacle =\n    barnacle.local_epmd()\n\n  // Run the refresh function once\n  barnacle.run_once(barnacle)\n\n  // Continue your program...\n}\n```\n\n## Strategies\n\nBarnacle ships with a few built-in strategies, but you can also supply your own by\nproviding a single callback function.\n\n### Local EPMD\n\nDiscover nodes using the local EPMD. This will automatically attempt to connect to\nnodes that are running on the same machine.\n\n```gleam\nimport barnacle\n\npub fn main() {\n  barnacle.local_epmd()\n  |&gt; barnacle.start\n}\n```\n\n### Remote EPMD\n\nConnect to nodes using a known list.\n\n```gleam\nimport barnacle\nimport gleam/erlang/atom\n\npub fn main() {\n  barnacle.epmd(\n    [&quot;node1@192.168.1.1&quot;, &quot;node2@192.168.1.2&quot;]\n    |&gt; list.map(atom.create_from_string),\n  )\n  |&gt; barnacle.start\n}\n```\n\n### DNS\n\nDiscover nodes using DNS.\n\nThe first argument is the basename of the node. Currently, Barnacle only supports\nconnecting to nodes with the same basename.\n\nBarnacle provides a helper function to get the basename of the current node.\n\n```gleam\nimport barnacle\n\npub fn main() {\n  let assert Ok(basename) = barnacle.get_node_basename(node.self())\n  barnacle.dns(\n    basename,\n    // The hostname to query against\n    &quot;my_app.example.com&quot;,\n  )\n  |&gt; barnacle.start\n}\n```\n\n### Creating a custom strategy\n\nYou can create your own strategy using the `new_strategy` function.\n\n```gleam\nimport barnacle\n\npub fn main() {\n  barnacle.new_strategy(\n    // Discover nodes\n    fn() {\n      Ok([])\n    },\n  )\n  |&gt; barnacle.custom\n  |&gt; barnacle.start\n}\n```\n\nYou can also supply your own functions for connecting, disconnecting, and listing nodes.\nIf these are not supplied, the built-in functions will be used.\n\nThis may be useful if you want to use an alternative to Distributed Erlang, such as\n[Partisan](https://github.com/lasp-lang/partisan).\n\n```gleam\nimport barnacle\n\npub fn main() {\n  barnacle.new_strategy(my_discover_function)\n  |&gt; barnacle.with_connect_nodes_function(my_connect_function)\n  |&gt; barnacle.with_disconnect_nodes_function(my_disconnect_function)\n  |&gt; barnacle.with_list_nodes_function(my_list_function)\n  |&gt; barnacle.custom\n  |&gt; barnacle.start\n}\n```\n\nYou can find more information about creating custom strategies in the\n[docs](/barnacle/barnacle.html#custom).\n\n## Events\n\nYou can provide a custom subject to receive events from your barnacle.\nThis will be notified whenever the barnacle refreshes, is paused, or is shutdown.\n\n```gleam\nimport barnacle\nimport gleam/erlang/process\n\npub fn main() {\n  let self = process.new_subject()\n\n  // Configure your Barnacle\n  let barnacle =\n    barnacle.local_epmd()\n      |&gt; barnacle.with_poll_interval(15_000)\n      |&gt; barnacle.with_name(&quot;my_barnacle&quot;)\n      |&gt; barnacle.with_listener(self)\n\n  // Start the actor\n  barnacle.start(barnacle)\n\n  // Wait for the barnacle to refresh\n  process.sleep(10_000)\n  let assert Ok(barnacle.RefreshResponse(Ok(new_nodes))) = process.receive(self, 10_000)\n}\n```\n\n## Manual interactions\n\nYou can interact with your barnacle manually by sending messages to it.\n\n```gleam\nimport barnacle\nimport gleam/erlang/process\nimport gleam/otp/actor\n\npub fn main() {\n  // Start a barnacle\n  let barnacle_subject =\n    barnacle.local_epmd()\n    |&gt; barnacle.start\n\n  let assert Ok(_) = barnacle.pause(barnacle_subject, 1000)\n  let assert Ok(_) = barnacle.refresh(barnacle_subject, 10_000)\n  let assert Ok(_) = barnacle.shutdown(barnacle_subject, 1000)\n}\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/barnacle&gt;.\n\n## Development\n\n```sh\ngleam run   # Run the project\ngleam test  # Run the tests\n```\n\nIf you would like to contribute, please open an issue or a PR. New strategies are\nwelcome, though try to keep dependencies to a minimum.\n\n## TODO\n\n- [ ] Tests!\n- [ ] Add new strategies\n  - [ ] Kubernetes\n  - [ ] Kubernetes with DNS\n  - [ ] Multicast UDP gossip\n  - [ ] `.hosts.erlang` file\n- [ ] Add a Fly.io example\n\n## With thanks\n\nA lot of inspiration came from the following projects:\n\n- [libcluster](https://github.com/bitwalker/libcluster)\n- [nessie_cluster](https://github.com/ckreiling/nessie_cluster)\n\nThanks!\n","url":"index.html"},{"doc":"barnacle","title":"Barnacle","content":"pub opaque type Barnacle(error)\n The entrypoint for creating a barnacle. Generic over the error returned by\n strategy-specific functions.\n\nSynonyms:\nbarnacle.Barnacle\nbarnacle Barnacle","url":"barnacle.html#Barnacle"},{"doc":"barnacle","title":"BarnacleResponse","content":"pub type BarnacleResponse(error) {\n  RefreshResponse(RefreshResult(error))\n  PauseResponse(Nil)\n  ShutdownResponse(Nil)\n}\n A response from a barnacle actor. This will be sent to the listener subject.\nRefreshResponse(RefreshResult(error))\n\n\nPauseResponse(Nil)\n\n\nShutdownResponse(Nil)\n\n\nSynonyms:\nbarnacle.BarnacleResponse\nbarnacle BarnacleResponse","url":"barnacle.html#BarnacleResponse"},{"doc":"barnacle","title":"Message","content":"pub opaque type Message(error)\n A message that can be sent to a barnacle actor.\n\nSynonyms:\nbarnacle.Message\nbarnacle Message","url":"barnacle.html#Message"},{"doc":"barnacle","title":"NodeDisconnectError","content":"pub type NodeDisconnectError {\n  FailedToDisconnect\n  LocalNodeIsNotAlive\n}\n An error that can occur when disconnecting from a node.\nFailedToDisconnect\n\n\nLocalNodeIsNotAlive\n\n\nSynonyms:\nbarnacle.NodeDisconnectError\nbarnacle NodeDisconnectError","url":"barnacle.html#NodeDisconnectError"},{"doc":"barnacle","title":"RefreshError","content":"pub type RefreshError(error) {\n  StrategyError(error)\n  ConnectError(List(#(atom.Atom, node.ConnectError)))\n  DisconnectError(List(#(atom.Atom, NodeDisconnectError)))\n}\n The error type for a barnacle refresh.\nStrategyError(error)\n\n\nConnectError(List(#(atom.Atom, node.ConnectError)))\n\n\nDisconnectError(List(#(atom.Atom, NodeDisconnectError)))\n\n\nSynonyms:\nbarnacle.RefreshError\nbarnacle RefreshError","url":"barnacle.html#RefreshError"},{"doc":"barnacle","title":"RefreshResult","content":"pub type RefreshResult(error) =\n  Result(List(atom.Atom), RefreshError(error))\n The result of a barnacle refresh.\n\nSynonyms:\nbarnacle.RefreshResult\nbarnacle RefreshResult","url":"barnacle.html#RefreshResult"},{"doc":"barnacle","title":"Strategy","content":"pub opaque type Strategy(error)\n A custom strategy for discovering and connecting to nodes.\n\nSynonyms:\nbarnacle.Strategy\nbarnacle Strategy","url":"barnacle.html#Strategy"},{"doc":"barnacle","title":"child_spec","content":"pub fn child_spec(\n  barnacle: Barnacle(a),\n  parent: Subject(Subject(Message(a))),\n) -&gt; ChildSpec(Message(a), b, b)\n Create a child spec for your barnacle for use in a supervision tree.\nSynonyms:\nbarnacle.child_spec\nbarnacle child_spec","url":"barnacle.html#child_spec"},{"doc":"barnacle","title":"custom","content":"pub fn custom(strategy: Strategy(a)) -&gt; Barnacle(a)\n Create a barnacle with a custom strategy. This requires at least one function\n to discover nodes.\n\n You can also supply your own functions for connecting, disconnecting, and\n listing nodes. If these are not supplied, the default Distributed Erlang\n functions will be used.\n\n Barnacles are generic over their error type, known as the strategy error.\n Your strategy functions may use this to return custom errors.\nSynonyms:\nbarnacle.custom\nbarnacle custom","url":"barnacle.html#custom"},{"doc":"barnacle","title":"dns","content":"pub fn dns(\n  basename: String,\n  hostname_query: String,\n) -&gt; Barnacle(Nil)\n Create a barnacle that discovers nodes using DNS. The first argument is the\n basename of the nodes. Currently, Barnacle only supports connecting to nodes\n with the same basename.\n\n The second argument is the hostname to query against. Both A and AAAA records\n will be queried.\nSynonyms:\nbarnacle.dns\nbarnacle dns","url":"barnacle.html#dns"},{"doc":"barnacle","title":"epmd","content":"pub fn epmd(nodes: List(Atom)) -&gt; Barnacle(Nil)\n Create a barnacle that connects to a known list of nodes. These may be\n remote nodes, or nodes on the same hostname as the current node.\nSynonyms:\nbarnacle.epmd\nbarnacle epmd","url":"barnacle.html#epmd"},{"doc":"barnacle","title":"get_node_basename","content":"pub fn get_node_basename(node: Node) -&gt; Result(String, Nil)\n Get the basename of a node.\n\n Useful for getting the basename of the current node for use with the DNS\n strategy.\n\n Example:\n \n import barnacle\n import gleam/erlang/node\n\n pub fn main() {\n   let assert Ok(basename) = barnacle.get_node_basename(node.self())\n }\n \nSynonyms:\nbarnacle.get_node_basename\nbarnacle get_node_basename","url":"barnacle.html#get_node_basename"},{"doc":"barnacle","title":"local_epmd","content":"pub fn local_epmd() -&gt; Barnacle(Nil)\n Create a barnacle that uses the local EPMD to discover nodes. This will discover\n nodes on the same hostname as the current node.\nSynonyms:\nbarnacle.local_epmd\nbarnacle local_epmd","url":"barnacle.html#local_epmd"},{"doc":"barnacle","title":"new_strategy","content":"pub fn new_strategy(\n  discover_nodes: fn() -&gt; Result(List(Atom), a),\n) -&gt; Strategy(a)\n Create a new custom strategy with a single function to discover nodes.\nSynonyms:\nbarnacle.new_strategy\nbarnacle new_strategy","url":"barnacle.html#new_strategy"},{"doc":"barnacle","title":"pause","content":"pub fn pause(subject: Subject(Message(a)), timeout: Int) -&gt; Nil\n Pause a barnacle actor. Sending a [`refresh`](#refresh) will restart the actor.\nSynonyms:\nbarnacle.pause\nbarnacle pause","url":"barnacle.html#pause"},{"doc":"barnacle","title":"refresh","content":"pub fn refresh(\n  subject: Subject(Message(a)),\n  timeout: Int,\n) -&gt; Result(List(Atom), RefreshError(a))\n Refresh a barnacle actor. This will attempt to connect to new nodes, and\n disconnect from nodes that are no longer available.\n\n This will reset the poll timer.\nSynonyms:\nbarnacle.refresh\nbarnacle refresh","url":"barnacle.html#refresh"},{"doc":"barnacle","title":"run_once","content":"pub fn run_once(\n  barnacle: Barnacle(a),\n) -&gt; Result(List(Atom), RefreshError(a))\n Run the refresh function once only. This will not start the actor, and will\n not do any polling.\n\n In this case, most configuration options are irrelevant and will be ignored.\nSynonyms:\nbarnacle.run_once\nbarnacle run_once","url":"barnacle.html#run_once"},{"doc":"barnacle","title":"shutdown","content":"pub fn shutdown(\n  subject: Subject(Message(a)),\n  timeout: Int,\n) -&gt; Nil\n Shutdown a barnacle actor. This will stop the actor, and stop any future\n refreshes.\nSynonyms:\nbarnacle.shutdown\nbarnacle shutdown","url":"barnacle.html#shutdown"},{"doc":"barnacle","title":"start","content":"pub fn start(\n  barnacle: Barnacle(a),\n) -&gt; Result(Subject(Message(a)), StartError)\n Start a barnacle actor. This will start the actor, and begin polling for\n nodes.\nSynonyms:\nbarnacle.start\nbarnacle start","url":"barnacle.html#start"},{"doc":"barnacle","title":"with_connect_nodes_function","content":"pub fn with_connect_nodes_function(\n  strategy: Strategy(a),\n  connect_nodes: fn(List(Atom)) -&gt;\n    Result(List(Atom), List(#(Atom, ConnectError))),\n) -&gt; Strategy(a)\n Add a custom node connection function to your strategy.\n Useful if you want to use an alternative to Distributed Erlang, such as\n [Partisan](https://github.com/lasp-lang/partisan).\nSynonyms:\nbarnacle.with_connect_nodes_function\nbarnacle with_connect_nodes_function","url":"barnacle.html#with_connect_nodes_function"},{"doc":"barnacle","title":"with_disconnect_nodes_function","content":"pub fn with_disconnect_nodes_function(\n  strategy: Strategy(a),\n  disconnect_nodes: fn(List(Atom)) -&gt;\n    Result(List(Atom), List(#(Atom, NodeDisconnectError))),\n) -&gt; Strategy(a)\n Add a custom node disconnection function to your strategy.\n Useful if you want to use an alternative to Distributed Erlang, such as\n [Partisan](https://github.com/lasp-lang/partisan).\nSynonyms:\nbarnacle.with_disconnect_nodes_function\nbarnacle with_disconnect_nodes_function","url":"barnacle.html#with_disconnect_nodes_function"},{"doc":"barnacle","title":"with_list_nodes_function","content":"pub fn with_list_nodes_function(\n  strategy: Strategy(a),\n  list_nodes: fn() -&gt; Result(List(Atom), a),\n) -&gt; Strategy(a)\n Add a custom node listing function to your strategy.\n Useful if you want to use an alternative to Distributed Erlang, such as\n [Partisan](https://github.com/lasp-lang/partisan).\nSynonyms:\nbarnacle.with_list_nodes_function\nbarnacle with_list_nodes_function","url":"barnacle.html#with_list_nodes_function"},{"doc":"barnacle","title":"with_listener","content":"pub fn with_listener(\n  barnacle: Barnacle(a),\n  listener: Subject(BarnacleResponse(a)),\n) -&gt; Barnacle(a)\n Set the listener subject for your barnacle. This will be notified whenever\n the barnacle refreshes, is paused, or is shutdown.\nSynonyms:\nbarnacle.with_listener\nbarnacle with_listener","url":"barnacle.html#with_listener"},{"doc":"barnacle","title":"with_name","content":"pub fn with_name(\n  barnacle: Barnacle(a),\n  name: String,\n) -&gt; Barnacle(a)\n Give the barnacle actor process a custom name.\nSynonyms:\nbarnacle.with_name\nbarnacle with_name","url":"barnacle.html#with_name"},{"doc":"barnacle","title":"with_poll_interval","content":"pub fn with_poll_interval(\n  barnacle: Barnacle(a),\n  poll_interval: Int,\n) -&gt; Barnacle(a)\n Set the poll interval for the barnacle actor. Defaults to 5000ms.\nSynonyms:\nbarnacle.with_poll_interval\nbarnacle with_poll_interval","url":"barnacle.html#with_poll_interval"},{"doc":"barnacle","title":"barnacle","content":"","url":"barnacle.html"}]);